//************************************************************************
// T6963.H
//
// LCD Gre und Grundeinstellungen angeben
//
// Holger Klabunde
// 18.12.2006
//************************************************************************
#ifndef t6963_h
#define t6963_h // Mehrfaches 'includen' vermeiden

#define NOP() _asm nop _endasm

// !!! only use ONE of the definitions below !!!
#define LCD_SIZE_240x64
//#define LCD_SIZE_240x128

#if defined (LCD_SIZE_240x64) && defined (LCD_SIZE_240x128)
#  error (Define "240x64" OR "240x128" display in t6963.h. Not BOTH !)
#endif

#ifdef LCD_SIZE_240x64
 #define LCD_WIDTH	240
 #define LCD_HEIGHT	64

 #define CGRAM_OFFSET  0x02   //CGRAM Startaddr 0x1000 for 240x64
 #define CGRAM_START   0x14   //For 240x64
#endif

#ifdef LCD_SIZE_240x128
 #define LCD_WIDTH	240
 #define LCD_HEIGHT	128

 #define CGRAM_OFFSET  0x03   //CGRAM Startaddr 0x1800 for 240x128
 #define CGRAM_START   0x1C   //For 240x128
#endif


// !!! only use ONE of the definitions below !!!
//#define PIXPERBYTE_6 //pin FONT of the display connected to +5V
                     //use this for 6x8 builtin font of display
                
#define PIXPERBYTE_8  //pin FONT of the display connected to GROUND
                      //use this for 8x8 builtin font of display

#if defined (PIXPERBYTE_6) && defined (PIXPERBYTE_8)
#  error (Define "6 Bit" OR "8 Bit" fontwidth in t6963.h. Not BOTH !)
#endif

#ifdef PIXPERBYTE_6       // 6 bits per character for builtin font
 #define PIXPERBYTE    6  // than also 6 pixel per byte for graphic
#endif

#ifdef PIXPERBYTE_8      // 8 bits per character for builtin font
 #define PIXPERBYTE    8 // than also 8 pixel per byte for graphic
#endif

#define BYTES_PER_ROW   LCD_WIDTH / PIXPERBYTE     // how many bytes per row on screen
#define DISPLAYBYTES    (unsigned int)(BYTES_PER_ROW * LCD_HEIGHT) // how many bytes for a screen

#define COLS	      LCD_WIDTH/PIXPERBYTE	//Text cols
#define ROWS	      LCD_HEIGHT/8              //Text rows

#define T_BASE  (unsigned int)0 	//begin of textarea
#define T_END   (unsigned int)(T_BASE + ( BYTES_PER_ROW * LCD_HEIGHT / 8 )) //end of textarea

//#define G_BASE  (unsigned int) (T_END +1)    	      // !! this did not work !!
#ifdef LCD_SIZE_240x64
 #define G_BASE  (unsigned int)0x0200    	      // base address of first screen
#endif

#ifdef LCD_SIZE_240x128
 #define G_BASE  (unsigned int)0x0300    	      // changed for 240x128 displays, up to 640 chars !
#endif

#define G_END  	(unsigned int)G_BASE+DISPLAYBYTES-1   // end adress of first screen
#define G_BASE1 (unsigned int)G_BASE+DISPLAYBYTES     // base address of 2nd screen
#define G_END1 	(unsigned int)G_BASE1+DISPLAYBYTES-1  // end adress of 2nd screen


//#define BAR	0x3E  //Bar mit einem LeerPixel zur optischen Trennung
#define BAR	0x1F  //Bar mit einem LeerPixel zur optischen Trennung
                      //sieht einfach besser aus

#define WR	PORTCbits.RC7
#define WR_TRIS	TRISCbits.TRISC7
#define RD	PORTCbits.RC1
#define RD_TRIS	TRISCbits.TRISC1
#define A0	PORTCbits.RC0     //eigentlich C/D
#define A0_TRIS	TRISCbits.TRISC0  //eigentlich C/D

#define DATA_DIR_IN()	TRISB=0xFF
#define DATA_DIR_OUT()  TRISB=0
#define WR_DATA(a)	PORTB=(a)		
#define RD_DATA()	PORTB		

#define CHECK_STA0_STA1		0x03
#define CHECK_STA3		0x08

#define AUTO_INCREMENT		0xC0
#define ADRESS_SET		0x24
#define AUTO_WRITE		0xB0
//#define AUTO_READ		0xB1
#define AUTO_RESET		0xB2

#define DATA	0       //Daten vom/zum Display
#define COMMAND	1       //Kommando/BusyCheck zum/vom Display

#define READ_T6963()   { \
                        DATA_DIR_IN(); \
                        RD=0; \
                        NOP(); \
                        t6963_busy=RD_DATA(); \
                        RD=1; \
                       };

#define WRITE_T6963(b) { \
                        DATA_DIR_OUT();\
                        WR_DATA((b));\
                        WR=0;\
                        NOP(); \
                        WR=1;\
                       };

#define CHECK_BUSY(c) { \
                       A0=COMMAND; \
                       DATA_DIR_IN(); \
                       do{ \
                          RD=0; \
                          NOP(); \
                          t6963_busy=RD_DATA(); \
                          RD=1; \
                         }while((t6963_busy & (c)) != (c)); \
                      };

#define WRITE_T6963_COMMAND(d) { \
                                A0=COMMAND; \
                                WRITE_T6963((d)); \
                               };

#define WRITE_T6963_DATA(e) { \
                                A0=DATA; \
                                WRITE_T6963((e)); \
                               };

extern void InitDisplay(void);
extern void LCDPos(unsigned char zeile, unsigned char spalte);
extern void LCDChar(unsigned char ch);
extern void ClearScreen(void);
extern void MakeCGRam(void);
extern void SetPixelArray(void);

extern unsigned char cgram[];
extern unsigned char xpos,ypos;
extern unsigned char pixbuffer[];
extern unsigned char pixlen;
extern unsigned char t6963_busy;

#endif // t6963_h
